import os
from ..utils.context import Context
from ..utils.logger import loading_process, completed_process, show_help

def purge():
  r = Context.root_path
  loading_process(
    f"Purging packages at: {r}",
    os.system,
    args=[f"rm -rf {r}/python_modules"]
  )
  completed_process("\nPackages purged successfully")
  
def help():
  show_help(
    command='purge',
    description='Purges all python packages in your project',
    usage=[
      {
        'command': 'purge',
        'description': 'Removes all python packages in your project (using your virtual environment)'
      }
    ]
  )


class Colors:
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'

class ColorPrinter:
    @staticmethod
    def red(text):
        print(Colors.RED + text + Colors.END)

    @staticmethod
    def purple(text):
        print(Colors.PURPLE + text + Colors.END)

    @staticmethod
    def cyan(text):
        print(Colors.CYAN + text + Colors.END)

    @staticmethod
    def darkcyan(text):
        print(Colors.DARKCYAN + text + Colors.END)

    @staticmethod
    def blue(text):
        print(Colors.BLUE + text + Colors.END)

    @staticmethod
    def green(text):
        print(Colors.GREEN + text + Colors.END)

chalk = ColorPrinter()


import os, yaml

from ..utils.logger import show_help 
from ..utils.context import Context
from ..utils.utils import check_project

def build():
  config = Context.config
  root_path = Context.root_path
  args = Context.command['args']

  check_project(args)

  global inner_config
  app_relative_path = os.path.join(root_path, config['services'][args]['path'])
  try:
    with open(os.path.join(app_relative_path, 'reqs.yaml'), 'r') as stream:
      inner_config = yaml.safe_load(stream)
    os.system(
      f"cd {app_relative_path}\n{inner_config['scripts']['build']}"
    )
  except ResourceWarning as e:
    print(e)

def help():
  show_help(
    command='build',
    description='Executes the build script for the project.',
    usage=[
      {
        'command': 'build',
        'description': 'Executes the build script in your reqs.yaml file.'
      },
    ]
  )


from __future__ import print_function
from typing import Any, List
from threading import Thread
from .chalk import chalk
from .context import Context
import cursor
from spinners import Spinners
import os
import sys
import time
import codecs

os.sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

CLEAR_LINE = '\033[K'

def decode_utf_8_text(text):
    """Decodes the text from utf-8 format.
    
    Parameters
    ----------
    text : str
        Text to be decoded
    
    Returns
    -------
    str
        Decoded text
    """
    try:
        return codecs.decode(text, 'utf-8')
    except:
        return text


def encode_utf_8_text(text):
    """Encodes the text to utf-8 format
    
    Parameters
    ----------
    text : str
        Text to be encoded
    
    Returns
    -------
    str
        Encoded text
    """
    try:
        return codecs.encode(text, 'utf-8')
    except:
        return text

if sys.version_info.major == 2:
    get_coded_text = encode_utf_8_text
else:
    get_coded_text = decode_utf_8_text

def animate(frames, interval, name, iterations=2):
    """Animate given frame for set number of iterations.

    Parameters
    ----------
    frames : list
        Frames for animating
    interval : float
        Interval between two frames
    name : str
        Name of animation
    iterations : int, optional
        Number of loops for animations
    """
    while Context.args['c_process']:
        for i in range(iterations):
            for frame in frames:
                frame = get_coded_text(frame)
                output = "\r{0} {1}\n".format(frame, name)
                sys.stdout.write(f"\033[95m{output}\033[0m")
                sys.stdout.write("\n")
                sys.stdout.write("\033[F") # Cursor up one line
                sys.stdout.write("\033[F") # Cursor up one line
                #sys.stdout.write(CLEAR_LINE)
                
                sys.stdout.flush()
                time.sleep(0.001 * interval)

def _loading(cmd: str):
    cursor.hide()
    spinner = [spinner for spinner in Spinners][0]
    frames = spinner.value['frames']
    interval = spinner.value['interval']
    animate(frames, interval, cmd)
    cursor.show()

def loading_process(
    log: str,
    f,
    args: List[str],
):
    Context.args['c_process'] = True
    p_log = Thread(target=_loading, args=[log])
    p_install = Thread(target=f, args=args)
    p_log.start()
    p_install.start()
    p_install.join()
    Context.args['c_process'] = False
    p_log.join()




def completed_process(cmd):
    chalk.green(cmd)

def failed_process(cmd):
    chalk.red(cmd)

def show_help(
    command: str,
    description: str,
    usage: List[Any]
):
    def pretty_print_usage(usage: Any):
        print(
            f"""
            \033[95mpyclk\033[1m {usage['command']}\033[0m | \033[92m{usage['description']}\033[0m"""
        )

    print(
        f"""
        Command: 
            \033[94m{command}\033[0m
        Description: 
            \033[92m{description}\033[0m
        
        Usage:"""
    )
    [pretty_print_usage(u) for u in usage]
    print()
    


import os,yaml
from ..utils.logger import show_help
from ..utils.context import Context
import subprocess

def activate():
  root_path = Context.root_path
  output = subprocess.check_output("pip -V", shell=True).decode('utf-8')
  if not output.__contains__('python_modules'):
    print("Error: You must activate python_modules first")
    print("Hint: ")
    print(" source ./python_modules/bin/activate")
    print("")

def help():
  show_help(
    command='activate',
    description='Show if your python_modules is activated or not',
    usage=[
      {
        'command': 'activate',
        'description': 'Checks for your virtual environment'
      },
    ]
  )


import yaml.emitter
import yaml.serializer
import yaml.representer
import yaml.resolver


class IndentingEmitter(yaml.emitter.Emitter):
    def increase_indent(self, flow=False, indentless=False):
        """Ensure that lists items are always indented."""
        return super().increase_indent(
            flow=False,
            indentless=False,
        )


class PrettyDumper(
    IndentingEmitter,
    yaml.serializer.Serializer,
    yaml.representer.Representer,
    yaml.resolver.Resolver,
):
    def __init__(
        self,
        stream,
        default_style=None,
        default_flow_style=False,
        canonical=None,
        indent=None,
        width=None,
        allow_unicode=None,
        line_break=None,
        encoding=None,
        explicit_start=None,
        explicit_end=None,
        version=None,
        tags=None,
        sort_keys=True,
    ):
        IndentingEmitter.__init__(
            self,
            stream,
            canonical=canonical,
            indent=indent,
            width=width,
            allow_unicode=allow_unicode,
            line_break=line_break,
        )
        yaml.serializer.Serializer.__init__(
            self,
            encoding=encoding,
            explicit_start=explicit_start,
            explicit_end=explicit_end,
            version=version,
            tags=tags,
        )
        yaml.representer.Representer.__init__(
            self,
            default_style=default_style,
            default_flow_style=default_flow_style,
            sort_keys=sort_keys,
        )
        yaml.resolver.Resolver.__init__(self)




import os, yaml, sys
from src.utils.context import Context
from src.utils.chalk import chalk
from src.utils.utils import run_hooks


Context.root_path = os.getcwd()
Context.config_file_path = os.path.join(Context.root_path, 'pyclk.yaml')
Context.commands = getattr(sys.modules['src'], 'commands')

#########################
#                       #
#   @pending changes    #
#   load version        #
#                       #
#########################
try:
  ###################
  #                 #
  #   production    #
  #                 #
  ###################
  from version import __version__ as version
  Context.version = version
except Exception as e:
  ###################
  #                 #
  #   development   #
  #                 #
  ###################
  with open("reqs.yaml", "r") as f:
    Context.version = yaml.safe_load(f)["version"]

def read_config():
  try:
    with open(Context.config_file_path, 'r') as stream:
      Context.config = yaml.safe_load(stream)
  except:
    print("Error: Could not load config file.")

def read_args():
  command_and_args = []
  flags = []

  def _get_flags(v: str):
    return v.__contains__('-') or v.__contains__('--')

  for arg in sys.argv:
    if _get_flags(arg):
      flags.append(
        arg.split('--')[1] if arg.__contains__('--') 
        else arg.split('-')[1]
      )
    else:
      command_and_args.append(arg)
  
  Context.command['flags'] = flags
  
  if len(command_and_args) == 1:
    return (None, None)
  try:
    return (command_and_args[1], command_and_args[2])
  except:
    return (command_and_args[1], None)

def run_command(cmd, args):
  try:
    status = Context.get_command(cmd)()
    ###################
    #                 #
    #    Run hooks    #
    #                 #
    ###################
    run_hooks(cmd) if status else None
    Context.flush_flags()
  except Exception as e:
    chalk.red("Error: Command not found")

def main():
  """
    Execute validation checks,
    and parse config files
  """
  read_config()
  (command, project) = read_args()
  if command is None:
    chalk.green(f"PyCLK v{Context.version}")
    sys.exit(0)

  Context.command['args'] = project
  Context.command['command'] = command

  run_command(command, project)



import os,yaml, sys
from ..utils.dumper import PrettyDumper
from ..utils.context import Context
from ..utils.logger import completed_process, loading_process, failed_process, show_help
from spinners import Spinners
from threading import Thread
import subprocess
from typing import List


def _remove_packages_conflicts(current: List[str], new: List[str]) -> List[str]:
  """
    Removes packages that are already installed.
  """
  combined_list = current + new
  unique_packages = {}
  for package in combined_list:
      package_parts = package.split('==')
      package_name = package_parts[0]
      if package_name not in unique_packages:
          unique_packages[package_name] = package_parts[1]
  return [f'{package}=={version}' for package, version in unique_packages.items()]

def _remove_empty(v):
  return True if len(v) > 0 else False

def _add_to_state():
  """
    Reads reqs.yaml and adds the new package to 
    the reqs.packages list.
  """

  root_path = Context.root_path
  activate_script = os.path.join(root_path, "python_modules", "bin", "activate")
  with open(os.path.join(root_path,'reqs.yaml'), 'r') as stream:
    Context.project = yaml.safe_load(stream)
  output = subprocess.check_output(f"/bin/bash -c \"source {activate_script} && pip freeze\"", shell=True).decode('utf-8')
  packages = list(filter(_remove_empty, output.split('\n')))
  Context.project['packages'] = _remove_packages_conflicts(Context.project['packages'], packages)

  with open(os.path.join(root_path,'reqs.yaml'), 'w') as stream:
    yaml.dump(
      Context.project,
      stream,
      default_flow_style=False,
      sort_keys=False,
      indent=2,
      Dumper=PrettyDumper
    )

def install():
  root_path = Context.root_path
  with open(os.path.join(root_path, 'reqs.yaml'), 'r') as stream:
    Context.project = yaml.safe_load(stream)

  packages_list = (
    " ".join(Context.project['packages']) if Context.command['args'] is None
    else Context.command['args']
  )
  if not os.path.isdir(os.path.join(root_path, 'python_modules')):
    print("Creating python_modules folder...")
    state = os.system(f'cd {root_path}\npython3 -m venv python_modules')
    if state != 0:
      failed_process("[Error]: Failed to create python_modules folder")
      os.system(f"rm -rf {root_path}/python_modules 2> /dev/null")
      return False
    
  path_to_python_version = os.path.join(
    root_path,
    "python_modules",
    "lib"
  )
  python_version = os.listdir(path_to_python_version)[0]
  path_to_env_folder = os.path.join(
    root_path,
    "python_modules",
    "lib",
    python_version,
    "site-packages"
  )

  loading_process(
    log=(
      "Installing full project..." if Context.command['args'] is None
      else f"Installing {packages_list}..."
    ),
    f=os.system,
    args=[f"pip install --quiet {packages_list} --target {path_to_env_folder} --upgrade"]
  )
  
  _add_to_state() if Context.command['args'] is not None else None
  
  completed_process("\nProject installed successfully")
  return True


def help():
  show_help(
    command='install',
    description='Installs packages into the python_modules folder',
    usage=[
      {
        'command': 'install',
        'description': 'Installs packages into the python_modules folder'
      },
      {
        'command': 'install <packages>',
        'description': 'Installs specific packages only'
      }
    ]
  )
  


from .context import Context
from .chalk import chalk
import os

def check_project(project_name):  
  if project_name not in Context.config['services']:
    print("Hint: Remember to add your project to pyclk.yaml")
    raise ResourceWarning("Error: Project not found.")


def run_hooks(command_name):
  try:
    total_hooks = len(Context.config['hooks'][command_name])
    print("\n")
    print("Running hooks")
    for c, hook in enumerate(Context.config['hooks'][command_name]):
      chalk.cyan(f"[{c +1}/{total_hooks}]: {hook}")
      print("====================")
      os.system(f"{hook}")
      print("====================\n")
  except:
    # it's okay if there are no hooks
    pass
  print("\n")



import os
from setuptools import setup, find_packages

def get_version_unsafe():
  with open("reqs.yaml", "r") as f:
    lines = f.readlines()
    for id,line in enumerate(lines):
      if line.startswith("version"):
        return lines[id+1].strip().replace("\\n", "")

version = get_version_unsafe()

with open("version.py", "w") as f:
  f.write("__version__ = '{}'\n".format(version))

with open("README.md", "r") as fh:
  long_description = fh.read()


if __name__ == "__main__":
  print("Running setup.py")
  setup(
    name='pyclk',
    license='MIT',
    author='Sebastian Tuyu',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author_email='contact@sebastiantuyu.com',
    keywords='monorepo cli package manger',
    description='Python package to manage monorepo cli',
    version=version,
    scripts=[
      'cli.py',
      'version.py'
    ],
    install_requires=[
      'pyyaml',
      'spinners',
      'cursor'
    ],
    entry_points={
      'console_scripts': [
        'pyclk = cli:main'
      ]
    },
    packages=find_packages()
  )


class _ServiceContext:
  def __init__(self):
    self.config = {}
    self.root_path = ""
    self.config_file_path = ""
    self.command = {
      'args': [],
      'command': '',
      'flags': [],
    }
    self.project = {}
    self.args = {}
    self.commands = {}
    self.version = ""

  def get_command(self, cmd):
    _to_run = (
      'help' 
        if ('help' or 'h') in self.command['flags']
        else cmd
      )
    return getattr(
      getattr((self.commands), cmd),
    _to_run)
  
  def flush_flags(self):
    self.command['flags'] = []

Context = _ServiceContext()